@@ -1,7 +1,7 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #
 # Autopilot Functional Test Tool
-# Copyright (C) 2012-2013 Canonical
+# Copyright (C) 2012-2014 Canonical
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -46,33 +46,32 @@ root of the application introspection tree.
 
 """
 
-from __future__ import absolute_import
-
 import logging
-import os
-import six
 
+from fixtures import EnvironmentVariable
 from testscenarios import TestWithScenarios
 from testtools import TestCase
+from testtools.content import ContentType, content_from_stream
 from testtools.matchers import Equals
+from testtools.testcase import _ExpectedFailure
+from unittest.case import SkipTest
 
 from autopilot.application import (
     ClickApplicationLauncher,
-    get_application_launcher_wrapper,
     NormalApplicationLauncher,
     UpstartApplicationLauncher,
 )
-from autopilot.display import Display
+from autopilot.display import Display, get_screenshot_data
 from autopilot.globals import get_debug_profile_fixture
 from autopilot.input import Keyboard, Mouse
-from autopilot.introspection import (
-    get_proxy_object_for_existing_process,
-)
 from autopilot.keybindings import KeybindingsHelper
 from autopilot.matchers import Eventually
+from autopilot.platform import get_display_server
 from autopilot.process import ProcessManager
 from autopilot.utilities import deprecated, on_test_started
 from autopilot._timeout import Timeout
+from autopilot._logging import TestCaseLoggingFixture
+from autopilot._video import get_video_recording_fixture
 try:
     from autopilot import tracepoint as tp
     HAVE_TRACEPOINT = True
@@ -136,8 +135,14 @@ class AutopilotTestCase(TestWithScenarios, TestCase, KeybindingsHelper):
     def setUp(self):
         super(AutopilotTestCase, self).setUp()
         on_test_started(self)
-        self.useFixture(get_debug_profile_fixture()(self.addDetail))
-
+        self.useFixture(
+            TestCaseLoggingFixture(
+                self.shortDescription(),
+                self.addDetailUniqueName,
+            )
+        )
+        self.useFixture(get_debug_profile_fixture()(self.addDetailUniqueName))
+        self.useFixture(get_video_recording_fixture()(self))
         _lttng_trace_test_started(self.id())
         self.addCleanup(_lttng_trace_test_ended, self.id())
 
@@ -157,6 +162,8 @@ class AutopilotTestCase(TestWithScenarios, TestCase, KeybindingsHelper):
                 "Process manager backend unavailable, application snapshot "
                 "support disabled.")
 
+        self.addOnException(self._take_screenshot_on_failure)
+
     @property
     def process_manager(self):
         if self._process_manager is None:
@@ -246,22 +253,19 @@ class AutopilotTestCase(TestWithScenarios, TestCase, KeybindingsHelper):
         :keyword emulator_base: If set, specifies the base class to be used for
             all emulators for this loaded application.
 
-        :raises ValueError: if unknown keyword arguments are passed.
         :return: A proxy object that represents the application. Introspection
          data is retrievable via this object.
 
         """
-        _logger.info(
-            "Attempting to launch application '%s' with arguments '%s' as a "
-            "normal process",
-            application,
-            ' '.join(arguments)
-        )
-        launcher = self.useFixture(
-            NormalApplicationLauncher(self.addDetail, **kwargs)
-        )
+        launch_args = _get_application_launch_args(kwargs)
 
-        return self._launch_test_application(launcher, application, *arguments)
+        launcher = self.useFixture(
+            NormalApplicationLauncher(
+                case_addDetail=self.addDetailUniqueName,
+                **kwargs
+            )
+        )
+        return launcher.launch(application, arguments, **launch_args)
 
     def launch_click_package(self, package_id, app_name=None, app_uris=[],
                              **kwargs):
@@ -297,25 +301,18 @@ class AutopilotTestCase(TestWithScenarios, TestCase, KeybindingsHelper):
         :returns: proxy object for the launched package application
 
         """
-        if isinstance(app_uris, (six.text_type, six.binary_type)):
-            app_uris = [app_uris]
-        _logger.info(
-            "Attempting to launch click application '%s' from click package "
-            " '%s' and URIs '%s'",
-            app_name if app_name is not None else "(default)",
-            package_id,
-            ','.join(app_uris)
-        )
         launcher = self.useFixture(
-            ClickApplicationLauncher(self.addDetail, **kwargs)
+            ClickApplicationLauncher(
+                case_addDetail=self.addDetailUniqueName,
+                **kwargs
+            )
         )
-        return self._launch_test_application(launcher, package_id, app_name,
-                                             app_uris)
+        return launcher.launch(package_id, app_name, app_uris)
 
     def launch_upstart_application(self, application_name, uris=[], **kwargs):
         """Launch an application with upstart.
 
-        This method launched an application via the ``upstart-app-launch``
+        This method launched an application via the ``ubuntu-app-launch``
         library, on platforms that support it.
 
         Usage is similar to the
@@ -328,47 +325,14 @@ class AutopilotTestCase(TestWithScenarios, TestCase, KeybindingsHelper):
             all emulators for this loaded application.
 
         :raises RuntimeError: If the specified application cannot be launched.
-        :raises ValueError: If unknown keyword arguments are specified.
         """
-        if isinstance(uris, (six.text_type, six.binary_type)):
-            uris = [uris]
-        _logger.info(
-            "Attempting to launch application '%s' with URIs '%s' via "
-            "upstart-app-launch",
-            application_name,
-            ','.join(uris)
-        )
         launcher = self.useFixture(
-            UpstartApplicationLauncher(self.addDetail, **kwargs)
+            UpstartApplicationLauncher(
+                case_addDetail=self.addDetailUniqueName,
+                **kwargs
+            )
         )
-        return self._launch_test_application(launcher, application_name, uris)
-
-    # Wrapper function tying the newer ApplicationLauncher behaviour with the
-    # previous (to be depreciated) behaviour
-    def _launch_test_application(self, launcher_instance, application, *args):
-
-        dbus_bus = launcher_instance.dbus_bus
-        if dbus_bus != 'session':
-            self.patch_environment("DBUS_SESSION_BUS_ADDRESS", dbus_bus)
-
-        pid = launcher_instance.launch(application, *args)
-        process = getattr(launcher_instance, 'process', None)
-        application_name = getattr(
-            launcher_instance,
-            'dbus_application_name',
-            None
-        )
-
-        proxy_obj = get_proxy_object_for_existing_process(
-            pid=pid,
-            process=process,
-            dbus_bus=dbus_bus,
-            emulator_base=launcher_instance.emulator_base,
-            application_name=application_name,
-        )
-        proxy_obj.set_process(process)
-
-        return proxy_obj
+        return launcher.launch(application_name, uris)
 
     def _compare_system_with_app_snapshot(self):
         """Compare the currently running application with the last snapshot.
@@ -385,62 +349,56 @@ class AutopilotTestCase(TestWithScenarios, TestCase, KeybindingsHelper):
         finally:
             self._app_snapshot = None
 
-    def patch_environment(self, key, value):
-        """Patch the process environment, setting *key* with value *value*.
+    def take_screenshot(self, attachment_name):
+        """Take a screenshot of the current screen and adds it to the test as a
+        detail named *attachment_name*.
 
-        This patches os.environ for the duration of the test only. After
-        calling this method, the following should be True::
+        If *attachment_name* already exists as a detail the name will be
+        modified to remove the naming conflict
+        (i.e. using TestCase.addDetailUniqueName).
 
-            os.environ[key] == value
-
-        After the test, the patch will be undone (including deleting the key if
-        if didn't exist before this method was called).
-
-        .. note:: Be aware that patching the environment in this way only
-         affects the current autopilot process, and any processes spawned by
-         autopilot. If you are planing on starting an application from within
-         autopilot and you want this new application to read the patched
-         environment variable, you must patch the environment *before*
-         launching the new process.
-
-        :param string key: The name of the key you wish to set. If the key
-         does not already exist in the process environment it will be created
-         (and then deleted when the test ends).
-        :param string value: The value you wish to set.
+        Returns True if the screenshot was taken and attached successfully,
+        False otherwise.
 
         """
-        if key in os.environ:
-            def _undo_patch(key, old_value):
-                _logger.info(
-                    "Resetting environment variable '%s' to '%s'",
-                    key,
-                    old_value
-                )
-                os.environ[key] = old_value
-            old_value = os.environ[key]
-            self.addCleanup(_undo_patch, key, old_value)
-        else:
-            def _remove_patch(key):
-                try:
-                    _logger.info(
-                        "Deleting previously-created environment "
-                        "variable '%s'",
-                        key
-                    )
-                    del os.environ[key]
-                except KeyError:
-                    _logger.warning(
-                        "Attempted to delete environment key '%s' that doesn't"
-                        "exist in the environment",
-                        key
-                    )
-            self.addCleanup(_remove_patch, key)
-        _logger.info(
-            "Setting environment variable '%s' to '%s'",
-            key,
-            value
-        )
-        os.environ[key] = value
+        try:
+            image_content = content_from_stream(
+                get_screenshot_data(get_display_server()),
+                content_type=ContentType('image', 'png'),
+                buffer_now=True
+            )
+            self.addDetailUniqueName(attachment_name, image_content)
+            return True
+        except Exception as e:
+            logging.error(
+                "Taking screenshot failed: {exception}".format(exception=e)
+            )
+            return False
+
+    def _take_screenshot_on_failure(self, ex_info):
+        failure_class_type = ex_info[0]
+        if _considered_failing_test(failure_class_type):
+            self.take_screenshot("FailedTestScreenshot")
+
+    @deprecated('fixtures.EnvironmentVariable')
+    def patch_environment(self, key, value):
+        """Patch environment using fixture.
+
+        This function is deprecated and planned for removal in autopilot 1.6.
+        New implementations should use EnvironmenVariable from the fixtures
+        module::
+
+            from fixtures import EnvironmentVariable
+
+            def my_test(AutopilotTestCase):
+                my_patch = EnvironmentVariable('key', 'value')
+                self.useFixture(my_patch)
+
+        'key' will be set to 'value'.  During tearDown, it will be reset to a
+        previous value, if one is found, or unset if not.
+
+        """
+        self.useFixture(EnvironmentVariable(key, value))
 
     def assertVisibleWindowStack(self, stack_start):
         """Check that the visible window stack starts with the windows passed
@@ -506,24 +464,20 @@ class AutopilotTestCase(TestWithScenarios, TestCase, KeybindingsHelper):
 
     assertProperties = assertProperty
 
-    @deprecated(
-        "the 'app_type' argument to the launch_test_application method"
-    )
-    def pick_app_launcher(self, app_path):
-        """Given an application path, return an object suitable for launching
-        the application.
 
-        This function attempts to guess what kind of application you are
-        launching. If, for some reason the default implementation returns the
-        wrong launcher, test authors may override this method to provide their
-        own implemetnation.
+def _get_application_launch_args(kwargs):
+    """Returns a dict containing relevant args and values for launching an
+    application.
 
-        The default implementation calls
-        :py:func:`autopilot.application.get_application_launcher_wrapper`
+    Removes used arguments from kwargs parameter.
 
-        """
-        # default implementation is in autopilot.application:
-        return get_application_launcher_wrapper(app_path)
+    """
+    launch_args = {}
+    launch_arg_list = ['app_type', 'launch_dir', 'capture_output']
+    for arg in launch_arg_list:
+        if arg in kwargs:
+            launch_args[arg] = kwargs.pop(arg)
+    return launch_args
 
 
 def _get_process_snapshot():
@@ -570,3 +524,10 @@ def _ensure_uinput_device_created():
             "Failed to create Touch device for bug lp:1297595 workaround: "
             "%s" % str(e)
         )
+
+
+def _considered_failing_test(failure_class_type):
+    return (
+        not issubclass(failure_class_type, SkipTest)
+        and not issubclass(failure_class_type, _ExpectedFailure)
+    )
