@@ -1,7 +1,7 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #
 # Autopilot Functional Test Tool
-# Copyright (C) 2012-2013 Canonical
+# Copyright (C) 2012-2014 Canonical
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,19 +18,21 @@
 #
 
 
-from __future__ import absolute_import
-
+from fixtures import TempDir
+import glob
 import os
 import os.path
 import re
 from tempfile import mktemp
 from testtools import skipIf
-from testtools.matchers import Contains, Equals, MatchesRegex, Not
+from testtools.matchers import Contains, Equals, MatchesRegex, Not, NotEquals
 from textwrap import dedent
+from unittest.mock import Mock
 
 from autopilot import platform
-from autopilot.testcase import AutopilotTestCase
+from autopilot.matchers import Eventually
 from autopilot.tests.functional import AutopilotRunTestBase, remove_if_exists
+from autopilot._video import RMDVideoLogFixture
 
 
 class AutopilotFunctionalTestsBase(AutopilotRunTestBase):
@@ -89,6 +91,9 @@ class AutopilotFunctionalTestsBase(AutopilotRunTestBase):
         )
         self.assertThat(expected_footer, Equals(observed_footer))
 
+
+class FunctionalTestMain(AutopilotFunctionalTestsBase):
+
     def test_can_list_empty_test_dir(self):
         """Autopilot list must report 0 tests found with an empty test
         module."""
@@ -325,40 +330,39 @@ Loading tests from: %s
     @skipIf(platform.model() != "Desktop", "Only suitable on Desktop (VidRec)")
     def test_record_flag_works(self):
         """Must be able to record videos when the -r flag is present."""
+        video_dir = mktemp()
+        ap_dir = '/tmp/autopilot'
+        video_session_pattern = '/tmp/rMD-session*'
+        self.addCleanup(remove_if_exists, video_dir)
+        self.addCleanup(
+            remove_if_exists,
+            '%s/Dummy_Description.ogv' % (ap_dir)
+        )
+        self.addCleanup(remove_if_exists, ap_dir)
 
-        # The sleep is to avoid the case where recordmydesktop does not create
-        # a file because it gets stopped before it's even started capturing
-        # anything.
-        self.create_test_file(
-            "test_simple.py", dedent("""\
+        mock_test_case = Mock()
+        mock_test_case.shortDescription.return_value = 'Dummy_Description'
+        orig_sessions = glob.glob(video_session_pattern)
 
-            from autopilot.testcase import AutopilotTestCase
-            from time import sleep
+        video_logger = RMDVideoLogFixture(video_dir, mock_test_case)
+        video_logger.setUp()
+        video_logger._test_passed = False
 
-            class SimpleTest(AutopilotTestCase):
-
-                def test_simple(self):
-                    sleep(1)
-                    self.fail()
-            """)
+        # We use Eventually() to avoid the case where recordmydesktop does not
+        # create a file because it gets stopped before it's even started
+        # capturing anything.
+        self.assertThat(
+            lambda: glob.glob(video_session_pattern),
+            Eventually(NotEquals(orig_sessions))
         )
 
-        should_delete = not os.path.exists('/tmp/autopilot')
-        if should_delete:
-            self.addCleanup(remove_if_exists, "/tmp/autopilot")
-        else:
-            self.addCleanup(
-                remove_if_exists,
-                '/tmp/autopilot/tests.test_simple.SimpleTest.test_simple.ogv')
+        video_logger._stop_video_capture(mock_test_case)
 
-        code, output, error = self.run_autopilot(["run", "-r", "tests"])
-
-        self.assertThat(code, Equals(1))
-        self.assertTrue(os.path.exists('/tmp/autopilot'))
+        self.assertTrue(os.path.exists(video_dir))
         self.assertTrue(os.path.exists(
-            '/tmp/autopilot/tests.test_simple.SimpleTest.test_simple.ogv'))
-        if should_delete:
-            self.addCleanup(remove_if_exists, "/tmp/autopilot")
+            '%s/Dummy_Description.ogv' % (video_dir)))
+        self.assertFalse(os.path.exists(
+            '%s/Dummy_Description.ogv' % (ap_dir)))
 
     @skipIf(platform.model() != "Desktop", "Only suitable on Desktop (VidRec)")
     def test_record_dir_option_and_record_works(self):
@@ -493,6 +497,25 @@ Loading tests from: %s
         self.assertThat(code, Equals(0))
         self.assertThat(os.path.exists(video_file_path), Equals(False))
 
+    @skipIf(platform.model() != "Desktop", "Only suitable on Desktop (VidRec)")
+    def test_no_video_session_dir_saved_for_passed_test(self):
+        """RecordMyDesktop should clean up its session files in tmp dir."""
+        with TempDir() as tmp_dir_fixture:
+            dir_pattern = os.path.join(tmp_dir_fixture.path, 'rMD-session*')
+            original_session_dirs = set(glob.glob(dir_pattern))
+            get_new_sessions = lambda: \
+                set(glob.glob(dir_pattern)) - original_session_dirs
+            mock_test_case = Mock()
+            mock_test_case.shortDescription.return_value = "Dummy_Description"
+            logger = RMDVideoLogFixture(tmp_dir_fixture.path, mock_test_case)
+            logger.set_recording_dir(tmp_dir_fixture.path)
+            logger._recording_opts = ['--workdir', tmp_dir_fixture.path] \
+                + logger._recording_opts
+            logger.setUp()
+            self.assertThat(get_new_sessions, Eventually(NotEquals(set())))
+            logger._stop_video_capture(mock_test_case)
+        self.assertThat(get_new_sessions, Eventually(Equals(set())))
+
     @skipIf(platform.model() != "Desktop", "Only suitable on Desktop (VidRec)")
     def test_no_video_for_nested_testcase_when_parent_and_child_fail(self):
         """Test recording must not create a new recording for nested testcases
@@ -596,7 +619,7 @@ SyntaxError: invalid syntax
 
     def test_can_create_subunit_result_file(self):
         self.create_test_file(
-            "test_simple.py", dedent(u"""\
+            "test_simple.py", dedent("""\
 
             from autopilot.testcase import AutopilotTestCase
 
@@ -708,49 +731,25 @@ SyntaxError: invalid syntax
         self.assertThat(code, Equals(0))
         self.assertThat(output, Not(Contains('Running tests in random order')))
 
+    def test_get_test_configuration_from_command_line(self):
+        self.create_test_file(
+            'test_config.py', dedent("""\
 
-class AutopilotPatchEnvironmentTests(AutopilotTestCase):
+                from autopilot import get_test_configuration
+                from autopilot.testcase import AutopilotTestCase
 
-    def test_patch_environment_new_patch_is_unset_to_none(self):
-        """patch_environment must unset the environment variable if previously
-        was unset.
+                class Tests(AutopilotTestCase):
 
-        """
-
-        class PatchEnvironmentSubTests(AutopilotTestCase):
-
-            def test_patch_env_sets_var(self):
-                """Setting the environment variable must make it available."""
-                self.patch_environment("APABC321", "Foo")
-                self.assertThat(os.getenv("APABC321"), Equals("Foo"))
-
-        self.assertThat(os.getenv('APABC321'), Equals(None))
-
-        result = PatchEnvironmentSubTests("test_patch_env_sets_var").run()
-
-        self.assertThat(result.wasSuccessful(), Equals(True))
-        self.assertThat(os.getenv('APABC321'), Equals(None))
-
-    def test_patch_environment_existing_patch_is_reset(self):
-        """patch_environment must reset the environment back to it's previous
-        value.
-
-        """
-
-        class PatchEnvironmentSubTests(AutopilotTestCase):
-
-            def test_patch_env_sets_var(self):
-                """Setting the environment variable must make it available."""
-                self.patch_environment("APABC987", "InnerTest")
-                self.assertThat(os.getenv("APABC987"), Equals("InnerTest"))
-
-        self.patch_environment('APABC987', "OuterTest")
-        self.assertThat(os.getenv('APABC987'), Equals("OuterTest"))
-
-        result = PatchEnvironmentSubTests("test_patch_env_sets_var").run()
-
-        self.assertThat(result.wasSuccessful(), Equals(True))
-        self.assertThat(os.getenv('APABC987'), Equals("OuterTest"))
+                    def test_foo(self):
+                        c = get_test_configuration()
+                        print(c['foo'])
+            """)
+        )
+        code, output, error = self.run_autopilot(
+            ["run", "--config", "foo=This is a test", "tests"]
+        )
+        self.assertThat(code, Equals(0))
+        self.assertIn("This is a test", output)
 
 
 class AutopilotVerboseFunctionalTests(AutopilotFunctionalTestsBase):
@@ -908,13 +907,19 @@ class AutopilotVerboseFunctionalTests(AutopilotFunctionalTestsBase):
 
         self.assertThat(code, Equals(0))
         self.assertThat(
-            error, Contains(
+            error,
+            Contains(
                 "Starting test tests.test_simple.OuterTestCase."
-                "test_nested_classes"))
+                "test_nested_classes"
+            )
+        )
         self.assertThat(
-            error, Contains(
+            error,
+            Contains(
                 "Starting test tests.test_simple.InnerTestCase."
-                "test_produce_log_output"))
+                "test_produce_log_output"
+            )
+        )
 
     def test_can_enable_debug_output(self):
         """Verbose log must show debug messages if we specify '-vv'."""
