@@ -17,10 +17,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-from dbus import SessionBus
 import json
 import logging
-from mock import patch
 import os
 import re
 import subprocess
@@ -38,14 +36,14 @@ from testtools.matchers import (
     StartsWith,
 )
 from textwrap import dedent
-from six import StringIO
+from io import StringIO
 
 from autopilot import platform
 from autopilot.matchers import Eventually
 from autopilot.testcase import AutopilotTestCase
 from autopilot.tests.functional.fixtures import TempDesktopFile
-from autopilot.introspection.dbus import CustomEmulatorBase
-from autopilot.introspection import _connection_matches_pid
+from autopilot.introspection import CustomEmulatorBase
+from autopilot.introspection.qt import QtObjectProxyMixin
 from autopilot.display import Display
 
 
@@ -76,6 +74,40 @@ class IntrospectionFeatureTests(AutopilotTestCase):
             emulator_base=emulator_base,
         )
 
+    def test_can_get_custom_proxy_for_app_root(self):
+        """Test two things:
+
+        1) We can get a custom proxy object for the root object in the object
+           tree.
+
+        2) We can get a custom proxy object for an object in the tree which
+           contains characters which are usually disallowed in python class
+           names.
+        """
+        class WindowMockerApp(EmulatorBase):
+            @classmethod
+            def validate_dbus_object(cls, path, _state):
+                return path == b'/window-mocker'
+
+        # verify that the initial proxy object we get back is the correct type:
+        app = self.start_mock_app(EmulatorBase)
+        self.assertThat(type(app), Equals(WindowMockerApp))
+
+        # verify that we get the correct type from get_root_instance:
+        self.assertThat(
+            type(app.get_root_instance()),
+            Equals(WindowMockerApp)
+        )
+
+    def test_customised_proxy_classes_have_extension_classes(self):
+        class WindowMockerApp(EmulatorBase):
+            @classmethod
+            def validate_dbus_object(cls, path, _state):
+                return path == b'/window-mocker'
+
+        app = self.start_mock_app(EmulatorBase)
+        self.assertThat(app.__class__.__bases__, Contains(QtObjectProxyMixin))
+
     def test_can_select_custom_emulators_by_name(self):
         """Must be able to select a custom emulator type by name."""
         class MouseTestWidget(EmulatorBase):
@@ -319,15 +351,3 @@ class QMLCustomEmulatorTestCase(AutopilotTestCase):
                 [e[1] for e in result2.decorated.errors]
             )
         )
-
-
-class IntrospectionFunctionTests(AutopilotTestCase):
-
-    @patch('autopilot.introspection._connection_matches_pid')
-    @patch('autopilot.introspection._bus_pid_is_our_pid')
-    def test_connection_matches_pid_ignores_dbus_daemon(
-            self, bus_pid_is_our_pid, conn_matches_pid_fn):
-        _connection_matches_pid(SessionBus(), 'org.freedesktop.DBus', 123)
-
-        self.assertThat(bus_pid_is_our_pid.called, Equals(False))
-        self.assertThat(conn_matches_pid_fn.called, Equals(False))
