@@ -17,15 +17,13 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-from __future__ import absolute_import
-
 from datetime import datetime, time
-from mock import patch, Mock
-import six
 from testscenarios import TestWithScenarios
 from testtools import TestCase
 from testtools.matchers import Equals, IsInstance, NotEquals, raises
+
 import dbus
+from unittest.mock import patch, Mock
 
 from autopilot.introspection.types import (
     Color,
@@ -66,22 +64,17 @@ class PlainTypeTests(TestWithScenarios, TestCase):
         ('int32 -ve', dict(t=dbus.Int32, v=-3002050)),
         ('int64 +ve', dict(t=dbus.Int64, v=9223372036854775807)),
         ('int64 -ve', dict(t=dbus.Int64, v=-9223372036854775807)),
-        ('ascii string', dict(t=dbus.String, v=u"Hello World")),
-        ('unicode string', dict(t=dbus.String, v=u"\u2603")),
+        ('ascii string', dict(t=dbus.String, v="Hello World")),
+        ('unicode string', dict(t=dbus.String, v="\u2603")),
         ('bytearray', dict(t=dbus.ByteArray, v=b"Hello World")),
-        ('object path', dict(t=dbus.ObjectPath, v=u"/path/to/object")),
-        ('dbus signature', dict(t=dbus.Signature, v=u"is")),
+        ('object path', dict(t=dbus.ObjectPath, v="/path/to/object")),
+        ('dbus signature', dict(t=dbus.Signature, v="is")),
         ('dictionary', dict(t=dbus.Dictionary, v={'hello': 'world'})),
         ('double', dict(t=dbus.Double, v=3.1415)),
         ('struct', dict(t=dbus.Struct, v=('some', 42, 'value'))),
         ('array', dict(t=dbus.Array, v=['some', 42, 'value'])),
     ]
 
-    if not six.PY3:
-        scenarios.append(
-            ('utf8 string', dict(t=dbus.UTF8String, v=b"Hello World"))
-        )
-
     def test_can_construct(self):
         p = PlainType(self.t(self.v))
 
@@ -100,14 +93,9 @@ class PlainTypeTests(TestWithScenarios, TestCase):
     def test_str(self):
         """str(p) for PlainType must be the same as the pythonic type."""
         p = PlainType(self.t(self.v))
-        try:
-            expected = str(self.v)
-            observed = str(p)
-            self.assertEqual(expected, observed)
-        # in Python 2.x, str(u'\2603') *should* raise a UnicodeEncode error:
-        except UnicodeEncodeError:
-            if not six.PY2:
-                raise
+        expected = str(self.v)
+        observed = str(p)
+        self.assertEqual(expected, observed)
 
     def test_wait_for_raises_RuntimeError(self):
         """The wait_for method must raise a RuntimeError if it's called."""
@@ -307,13 +295,25 @@ class DateTimeTests(TestCase):
     def test_datetime_has_properties(self):
         dt = DateTime(1377209927)
 
-        self.assertThat(dt.timestamp, Equals(1377209927))
-        self.assertThat(dt.year, Equals(2013))
-        self.assertThat(dt.month, Equals(8))
-        self.assertThat(dt.day, Equals(22))
-        self.assertThat(dt.hour, Equals(22))
-        self.assertThat(dt.minute, Equals(18))
-        self.assertThat(dt.second, Equals(47))
+        self.assertTrue(hasattr(dt, 'timestamp'))
+        self.assertTrue(hasattr(dt, 'year'))
+        self.assertTrue(hasattr(dt, 'month'))
+        self.assertTrue(hasattr(dt, 'day'))
+        self.assertTrue(hasattr(dt, 'hour'))
+        self.assertTrue(hasattr(dt, 'minute'))
+        self.assertTrue(hasattr(dt, 'second'))
+
+    def test_datetime_properties_have_correct_values(self):
+        dt = DateTime(1377209927)
+        dt_with_tz = datetime.fromtimestamp(1377209927)
+
+        self.assertThat(dt.timestamp, Equals(dt_with_tz.timestamp()))
+        self.assertThat(dt.year, Equals(dt_with_tz.year))
+        self.assertThat(dt.month, Equals(dt_with_tz.month))
+        self.assertThat(dt.day, Equals(dt_with_tz.day))
+        self.assertThat(dt.hour, Equals(dt_with_tz.hour))
+        self.assertThat(dt.minute, Equals(dt_with_tz.minute))
+        self.assertThat(dt.second, Equals(dt_with_tz.second))
 
     def test_equality_with_datetime(self):
         dt1 = DateTime(1377209927)
@@ -328,9 +328,10 @@ class DateTimeTests(TestCase):
         self.assertThat(dt1, Equals(dt2))
 
     def test_equality_with_datetime_timestamp(self):
+        # DateTime no longer assumes UTC and uses local TZ.
         dt1 = DateTime(1377209927)
-        dt2 = datetime.utcfromtimestamp(1377209927)
-        dt3 = datetime.utcfromtimestamp(1377209928)
+        dt2 = datetime.fromtimestamp(1377209927)
+        dt3 = datetime.fromtimestamp(1377209928)
 
         self.assertThat(dt1, Equals(dt2))
         self.assertThat(dt1, NotEquals(dt3))
@@ -341,8 +342,12 @@ class DateTimeTests(TestCase):
         self.assertThat(dt1.datetime, IsInstance(datetime))
 
     def test_repr(self):
+        expected = repr_type(
+            u"DateTime({:%Y-%m-%d %H:%M:%S})".format(
+                datetime.fromtimestamp(1377209927)
+            )
+        )
         dt = DateTime(1377209927)
-        expected = repr_type('DateTime(2013-08-22 22:18:47)')
         observed = repr(dt)
         self.assertEqual(expected, observed)
 
@@ -773,13 +778,13 @@ class DBusIntrospectionObjectTests(TestCase):
 
         """
         DBusIntrospectionObject(
-            dict(foo=[0]),
-            '/some/dummy/path',
+            dict(foo=[0], id=[0, 42]),
+            b'/some/dummy/path',
             Mock()
         )
         error_logger.assert_called_once_with(
             "While constructing attribute '%s.%s': %s",
-            "DBusIntrospectionObject",
+            "ProxyBase",
             "foo",
             "Cannot create attribute, no data supplied"
         )
@@ -838,12 +843,12 @@ class TypeReprTests(TestCase):
         self.assertEqual(_boolean_repr, observed)
 
     def test_text_repr_handles_dbus_string(self):
-        unicode_text = u"plɹoʍ ollǝɥ"
+        unicode_text = "plɹoʍ ollǝɥ"
         observed = _text_repr(dbus.String(unicode_text))
         self.assertEqual(repr(unicode_text), observed)
 
     def test_text_repr_handles_dbus_object_path(self):
-        path = u"/path/to/some/object"
+        path = "/path/to/some/object"
         observed = _text_repr(dbus.ObjectPath(path))
         self.assertEqual(repr(path), observed)
 
